/*  
 created 29 Dec. 2008
 modified 30 Aug 2011
 by Tom Igoe
 
 This example code is in the public domain.
 
 Modified past recognition by Doug Coulter for real data aquisition and a useful output format.
 Begun 12/4/2013
 Counter 1 added 12/7/13  input is arudino pin 5, falling edge, with pullup

GPL V2 license - share and enjoy, but if you come up with
a cool new trick, pleasse send it to dcfusor@gmail.com so I can
put it up on the forums for all of us to share and enjoy.
(www.coultersmithing.com/forums)
*/

//   globals
#define ADCHNLS 6  // no ; after defines in arduino land
int AD[ADCHNLS];        // values read from the a/d
boolean digout = false;
unsigned long old_millis = 0;   // Z-1 for timing, rather than just delaying
volatile unsigned long c1counts;    // total counts from counter 1, top half incremented by timer overflow
unsigned long c1z = 0; // previous count for subtraction
unsigned long curcount =  0; // what we'll send, this - last total
//////////////////////////////////////////////// subs called from setup
void t1set() // setup timer1 as a counter that interrupts on overflow
{
  TCCR1A = 0;
  TCCR1B = 6; // clock on falling edge (7 for rising)
  TCNT1 = 0; //zero out timer
  TIMSK1 = _BV(TOIE1);// sets the timer overflow interrupt enable bit
  }
/////////////////////////////////////////////////////////
//////// manual isr setup, you bet, saves a whole library
void t1OVF() // declare routine so the below knows its address
{
 c1counts += 0x00010000; // add one to top half of longword
 // the low 16 bits took care of wrapping to zero on their own
}
// actual isr just has room for a call at vector
ISR(TIMER1_OVF_vect)
{
 t1OVF();
} 
////////////////////////////////////////////////
/* *********************************************
 Always hit on any reset - soft, hard, serial port connect...
*/
void setup() {
  // initialize serial communications
  Serial.begin(115200); // 115,200? Yeah!
//  Serial.print("Setup reached:"); // every reset of any kind
 
  pinMode(12,OUTPUT);  // scope for loop timing etc
  pinMode(5,INPUT); // counter input @@@ INPUT_PULLUP later?
  //@@@ their bug - INPUT_PULLUP not recognized like dox say
  digitalWrite(5, HIGH); // turns on pullup resistor
  t1set(); // setup timer 1 after ensuring high input

//  Serial.println("\tSetup end"); // every reset of any kind
}
////////////////////////////////////////////////
////////////////////////////////////////////////

//////////////////////////////////////////////// subs used in loop (main)
void soft_reset ()
{ // so we can reset timers and so on under host control
 asm volatile ("  jmp 0"); 
} // this really is a hard reset, but coming from software
////////////////////////////////////////////////

////////////////////////////////////////////////

////////////////////////////////////////////////
void readAD()
{
 byte chnl;  // Analog input pin 
 byte pass;

/*  for (chnl=0; chnl < ADCHNLS; chnl++)
  {
   AD[chnl] = 0; // clear out old
  }
  for (pass=0; pass < 2; pass++)
  {
 */ 
   for (chnl=0; chnl < ADCHNLS; chnl++)
   {
    delayMicroseconds(10); // Maybe not required, it's slow enough.
    AD[chnl]  = analogRead(chnl); // delay required?
   }
 /* } // for pass

  for (chnl=0; chnl < ADCHNLS; chnl++)
  {
   AD[chnl] = AD[chnl] >> 1
   ; // rescale
  }
*/
  
}
////////////////////////////////////////////////
void readCounts()
{
/* Disable interrupts */
noInterrupts();
/* Read TCNT1 into c1counts low part */
c1counts &= 0xffff0000;
c1counts |= TCNT1;
curcount = c1counts - c1z; // just the new ones for this time interval
c1z = c1counts; // for next time
/* Restore global interrupt flag */
interrupts(); 
}
////////////////////////////////////////////////
void printOut()
{
 Serial.print (millis());
 byte i;
 for (i=0; i < ADCHNLS; i++)
 {
  Serial.print ("\t");  // some delimiter
  Serial.print (AD[i]);
 }
 Serial.print ("\t");
 Serial.print(curcount); // new (only) counts from hardware counter
 Serial.println(""); // arduino windows CRLF for now (PITA for linux)
}
//////////////////////////////////////////////// main loop, called repeatedly from base arduino code (about 15 us in their code per)
void loop() {
 if (millis() - old_millis >= 100)
 {
  old_millis = millis(); // reset time-base 
  readAD(); // read all the channels
  readCounts(); // get timer 1 counts to c1counts long
  printOut(); // tell the world what we see
 }
 
///* 
 if (digout) // show time between loop calls, longer time when we do work - debug
 
 {
  digitalWrite(12,HIGH); // on the LED
  } else
  {
   digitalWrite(12, LOW);   // off the LED 
  } 
  digout = !digout; //toggle every time through
 // */
 
  // I'll be adding other commands to this, and it will probably
  // become a switch plus state machine kinda thing...
  if ((Serial.available() > 0) && ('!' == Serial.read())) soft_reset(); // reset clock if ! sent
  
  
} // end main loop

